package br.com.rhino.user.service;

import static br.com.rhino.user.entity.User.deleteById;
import static br.com.rhino.user.entity.User.find;
import static br.com.rhino.user.entity.User.findAll;
import static br.com.rhino.user.entity.User.findByIdOptional;

import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.Optional;
import java.util.Set;
import java.util.stream.Collectors;
import javax.transaction.Transactional;

import br.com.rhino.user.dto.UserDTO;
import br.com.rhino.user.entity.Role;
import br.com.rhino.user.entity.User;
import br.com.rhino.user.exception.UserNotFoundException;
import br.com.rhino.user.mapper.RoleMapper;
import br.com.rhino.user.mapper.UserMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Service
public class UserService {

    private final UserMapper userMapper;
    private final RoleMapper roleMapper;

    @Autowired
    public UserService(final UserMapper userMapper, final RoleMapper roleMapper) {
        this.userMapper = userMapper;
        this.roleMapper = roleMapper;
    }

    @Transactional
    public UserDTO save(final UserDTO userDTO) {
        final User userPO = userMapper.fromResource(userDTO);
        userPO.persist();
        return userMapper.toResource(userPO);
    }

    @Transactional
    public UserDTO update(final Long id, final UserDTO userDTO) throws UserNotFoundException {
        final Optional<User> userPO = findByIdOptional(id);
        final Set<Role> roles = getRolesFromUserDTO(userDTO);

        return userPO.map(user -> {
            user.email(userDTO.email)
                    .username(userDTO.username)
                    .password(userDTO.password)
                    .email(userDTO.email)
                    .roles(roles);
            user.persist();
            return userMapper.toResource(user);
        }).orElseThrow(() -> new UserNotFoundException("Usuário não encontrado id: " + id));
    }

    public UserDTO findByUsername(final String username) throws UserNotFoundException {
        return Optional.ofNullable(find("username", username).singleResult())
                .map(user -> userMapper.toResource((User) user))
                .orElseThrow(() -> new UserNotFoundException("Usuário não encontrado username: " + username));
    }

    public UserDTO findById(final Long id) throws UserNotFoundException {
        return findByIdOptional(id)
                .map(user -> userMapper.toResource((User) user))
                .orElseThrow(() -> new UserNotFoundException("Usuário não encontrado id: " + id));
    }

    public List<UserDTO> findAllUsers() {
        return Optional.ofNullable(findAll().list())
                .stream()
                .flatMap(Collection::stream)
                .map(user -> userMapper.toResource((User) user))
                .collect(Collectors.toList());
    }

    @Transactional
    public void delete(final Long id) {
        deleteById(id);
    }

    private Set<Role> getRolesFromUserDTO(final UserDTO userDTO) {
        final Set<Role> roles = Optional.ofNullable(userDTO.roles)
                .orElse(Collections.emptySet())
                .stream().map(dto -> roleMapper.fromResource(dto))
                .collect(Collectors.toSet());
        return roles;
    }
}
